package edu.uconn.engr.dna.isoem;

import edu.uconn.engr.dna.format.CoordinatesMapper;
import edu.uconn.engr.dna.format.Isoform;
import edu.uconn.engr.dna.format.Isoforms;
import edu.uconn.engr.dna.util.Converter;
import edu.uconn.engr.dna.util.IntervalVisitor;
import edu.uconn.engr.dna.util.Intervals;
import edu.uconn.engr.dna.util.TokenizerWithReplace;
import java.util.ArrayList;
import java.util.List;
//import edu.uconn.engr.dna.util.Utils;

public class SamIsoformToGenomeConverter implements Converter<String, String> {
	private static final int READ_PAIRED_FLAG = 0x1;
	private static final int SEQUENCE_UNMAPPED = 0x4;
	private static final int MATE_UNMAPPED = 0x8;
	private static final int READ_STRAND_FLAG = 0x10;
	private static final int MATE_STRAND_FLAG = 0x20;

	private Isoforms isoforms;
	private TokenizerWithReplace rTok;
	private VisitorForGenomeCigar visitorForGenomeCigar;
        private int polyAtail;

	public SamIsoformToGenomeConverter(Isoforms isoforms) {
                this.polyAtail = 0;
		this.isoforms = isoforms;
		char separator = '\t';
		this.rTok = new TokenizerWithReplace(separator);
		visitorForGenomeCigar = new VisitorForGenomeCigar();
	}

	public SamIsoformToGenomeConverter(Isoforms isoforms, int polyAtail) {
                this.polyAtail = polyAtail;
		this.isoforms = isoforms;
		char separator = '\t';
		this.rTok = new TokenizerWithReplace(separator);
		visitorForGenomeCigar = new VisitorForGenomeCigar();
	}
	public String convert(String line) {
		// Parse line
		rTok.setLine(line);
		rTok.skipNextWithCopy(); // skip readName
		int flags = rTok.nextInt();
		if ((flags & SEQUENCE_UNMAPPED) != 0) { //  read is not mapped
			return null;
		}
		String referenceSequenceName = rTok.nextString(); // (isoform)
		Isoform isoform = isoforms.getValue(referenceSequenceName);
		if (isoform == null) {
			return null;
		}
		int newFlags = getNewFlags(flags, isoform);
		rTok.update(newFlags); // add updated flags
		rTok.update(isoform.getChromosome()); // add chromosome

		int alignmentStart = rTok.nextInt();
		rTok.skipNext(); // skip <MAPQ>
		int mapqs = rTok.getLastTokenStart(); //but keep track of location
		int mapqe = rTok.getLastTokenEnd();
//indel handling
                String transcriptCigar=rTok.nextString();
		//rTok.skipNext();
		//int readLength = getReadLength(line, rTok.getLastTokenStart(), rTok.getLastTokenEnd());
              int readLength = getReadLength(transcriptCigar);
//		System.out.println(referenceSequenceName);
//		System.out.println("before strand translation: stran, aln_start, iso length, read length, cigar, isoform start position");
//		System.out.println(isoform.getStrand());
//		System.out.println(alignmentStart);
//		System.out.println(isoform.length());
//		System.out.println(readLength);
//		System.out.println(transcriptCigar);
//		System.out.println(isoform.getExons().getStart(0)+" "+isoform.getExons().getEnd(0));
//		System.out.println(isoform.getExons().getStart(1)+" "+isoform.getExons().getEnd(1));
//		System.out.println(isoform.getExons().getStart(2)+" "+isoform.getExons().getEnd(2));

		alignmentStart = getReadIsoformPositionInPlusStrand(alignmentStart, isoform, readLength);
//		System.out.println("alignment start adjusted for strand");
//		System.out.println(alignmentStart);
		int genomeAlignmentStart = getGenomePosition(alignmentStart, isoform);
                int BasesInPolyA=CoordinatesMapper.numberOfBasesMappedToPolyAtail(isoform.getExons(),alignmentStart, readLength, polyAtail, isoform.getStrand());
//		System.out.println("BasesInPolyA");
//		System.out.println(BasesInPolyA);
                if (BasesInPolyA == readLength)
                    return null;
		rTok.update(genomeAlignmentStart); // add updated alignment start
		rTok.update(line, mapqs, mapqe); // add MAPQ unchanged
//indel handling
                CharSequence genomeCigar = getGenomeCigar(readLength, alignmentStart, isoform,transcriptCigar,BasesInPolyA);
		rTok.update(genomeCigar);

		rTok.skipNextWithCopy(); // skip <MRNM>
		int matePosition = rTok.nextInt();
		if (matePosition != 0) {
			matePosition = getReadIsoformPositionInPlusStrand(matePosition, isoform, readLength);
			matePosition = getGenomePosition(matePosition, isoform);
		}
		rTok.update(matePosition);
		rTok.skipNextWithCopy(); // skip <ISIZE>

		if (isoform.getStrand() == '-') {
			rTok.replaceNextTokenWithReverseComplement(); // sequence
			rTok.replaceNextTokenWithReverse(); // quality //should add one for cigar
			return rTok.getNewLine("XS:A:-");
		} else {
			return rTok.getNewLine("XS:A:+");
		}
	}

	public static int getReadIsoformPositionInPlusStrand(int alignmentStart, Isoform isoform, int readLength) {
		return getReadIsoformPositionInPlusStrand(alignmentStart, isoform.getStrand(),
				isoform.length(), readLength);
	}

	public static int getReadIsoformPositionInPlusStrand(int alignmentStart, char strand,
			int isoformLength, int readLength) {
		
//		System.out.println("isoformLength - alignmentStart - readLength + 2 = "+isoformLength+" - "+alignmentStart+" - "+readLength+" + 2");
		if (strand == '-') {
			return isoformLength - alignmentStart - readLength + 2;
		}
		return alignmentStart;
		
	}

	private int getGenomePosition(int alignmentStart, Isoform isoform) {
		return (int) CoordinatesMapper.isoformToGenomeCoordinate(isoform.getExons(), alignmentStart, polyAtail, isoform.getStrand());
	}

	private int getNewFlags(int flags, Isoform isoform) {
		if (isoform.getStrand() == '-') {
			boolean readPaired = ((flags & READ_PAIRED_FLAG) != 0)
					&& ((flags & SEQUENCE_UNMAPPED) == 0)
					&& ((flags & MATE_UNMAPPED) == 0);
			flags ^= READ_STRAND_FLAG; // flip read strand
			if (readPaired) {
				flags ^= MATE_STRAND_FLAG; // flip mate strand
			}
		}
		return flags;
	}

        private CharSequence buildCigar(List<Integer> cigarElemsLen, List<Character> cigarElems, char strand, int basesInPolyAtail) {
		final StringBuilder cigar = new StringBuilder();
                int startAt = cigarElems.size()-1;

                // polyA tail clipping do it while processing the last element of the CIGAR if
                // transcript is on +ve strand. Case of -ve strand will be done in the iteration i = 0
                // in the loop below (first element in the CIGAR.
                if (strand == '+') {
                    if (cigarElems.get(startAt) == 'S') {
                           cigar.append((int) cigarElemsLen.get(startAt) + basesInPolyAtail);
                           cigar.append('S');
                           startAt--;
                    }
                    else if (basesInPolyAtail > 0) {
                           cigar.append(basesInPolyAtail);
                           cigar.append('S');
                    }
                }
                if (cigarElems.get(startAt) == 'N')
                    startAt--;
                for (int i = startAt; i >= 0;  i--) {
//                   if (i > 0)  //bug fix for converting to pileup. samtools do not like CIGARs such as  3S34M4831N2I13M1S with I after N
//                       if (cigarElems.get(i) == 'I' && cigarElems.get(i-1) == 'N')
//                           return null;

                // polyA tail clipping do it while processing the first element of the CIGAR if
                // transcript is on -ve strand. Case of first element is not S will be done
                // at the end of loop below (after adding first element if CIGAR)

                    if (i == 0 && strand == '-' && cigarElems.get(i) == 'S') {
                           cigar.insert(0,'S');
                           cigar.insert(0,(int) cigarElemsLen.get(startAt) + basesInPolyAtail);
                           startAt--;
                           break;
                    }
                   int elemLen = Math.max(0,cigarElemsLen.get(i) - basesInPolyAtail);
                   basesInPolyAtail = Math.max(0,basesInPolyAtail-cigarElemsLen.get(i));
		     if (elemLen > 0) {
//	                   if (strand == '+') {
       	                cigar.insert(0,cigarElems.get(i));
              	         cigar.insert(0,(int) elemLen);
//                   		}
//			   else {
//		               cigar.append((int) elemLen);
//            	         cigar.append(cigarElems.get(i));
//			    }
			}
                   else if (i > 0) //elemLen = 0  //if the whole current element cigar will be clipped it was preceded by 'N'; skip the 'N'
                       if (cigarElems.get(i-1) == 'N')
                           i--; 
			
                    if (i == 0 && strand == '-' && cigarElems.get(i) != 'S' && basesInPolyAtail > 0) {
                           cigar.insert(0,'S');
                           cigar.insert(0,basesInPolyAtail);
                    }
                }

                return cigar;

        }

        

	private CharSequence getGenomeCigar(int readLength, int alignmentStart, Isoform isoform, String transcriptomeCigar, int BasesInPolyA) {
//method updated to account for indels in cigar
		final StringBuilder cigar = new StringBuilder();
                visitorForGenomeCigar.setCigar(cigar);
                visitorForGenomeCigar.setTranscriptomeCigar(transcriptomeCigar);
		List<Integer> cigarElemsLen = new ArrayList<Integer>();
		List<Character> cigarElems = new ArrayList<Character>();

//		CoordinatesMapper.visitGenomeIntervals(isoform.getExons(),
//				alignmentStart, alignmentStart + readLength - 1,
//				visitorForGenomeCigar);
                Intervals exons = isoform.getExons();
                int cigarLength =  transcriptomeCigar.length();
                int currentExon;
                int cigarIndex = 0;
                int accumCigarToTramscriptLength = 0;

                 int currentExonLength; 
                int accumExonLength = 0; 

                if (isoform.getStrand() == '-' && polyAtail > 0) {
                    currentExon = 1; //skip polyAtail Exon
                    accumExonLength = polyAtail; //(or lengnth of exon 1)
                }
                else 
                    currentExon = 0;

                currentExonLength = exons.length(currentExon);
                accumExonLength += currentExonLength;


		  while (accumExonLength < alignmentStart) {
                   currentExon++;
                   currentExonLength = exons.length(currentExon);
                   accumExonLength += currentExonLength;

		
		  }
		  accumExonLength -= (alignmentStart -1);
                while (cigarIndex < cigarLength) {
                    StringBuilder cigarElemLength = new StringBuilder();
                     while ((cigarIndex < cigarLength) && (Character.isDigit(transcriptomeCigar.charAt(cigarIndex)))){
                            cigarElemLength.append(transcriptomeCigar.charAt(cigarIndex));
                            cigarIndex++;
                    }
                    //int elemLen = Utils.parseInt(cigarElemLength,10,0,cigarElemLength.length()-1);
			int elemLen = Integer.parseInt(cigarElemLength.toString(),10);//,0,cigarElemLength.length()-1);

                    if (cigarIndex == cigarLength)
                        throw new IllegalArgumentException("Incorrect cigar string format" + transcriptomeCigar);
                    char cigarElement = transcriptomeCigar.charAt(cigarIndex);
                    cigarIndex++;
                    while (elemLen > 0) {
                        switch (cigarElement) {
                            case 'M':
                            case '=':
                            case 'X':
                            case 'D':
                                if ((accumCigarToTramscriptLength + elemLen) > (accumExonLength)) {
                                    if (currentExon == exons.length()-1)  //last exon
                                        throw new IllegalArgumentException("cigar string extends beyond transcript" + transcriptomeCigar);
                                    int nextElemLen = accumCigarToTramscriptLength + elemLen - accumExonLength;
					 int tempElemLen = elemLen - nextElemLen;
                                    cigarElemsLen.add(tempElemLen);
                                    cigarElems.add(cigarElement);
                                    accumCigarToTramscriptLength += tempElemLen;
                                    elemLen = nextElemLen;
                                }
                                else {
                                    cigarElemsLen.add(elemLen);
                                    cigarElems.add(cigarElement);
                                    accumCigarToTramscriptLength += elemLen;
                                    elemLen = 0;
                                }
                                break;
                            case 'S':
                                    cigarElemsLen.add(elemLen);
                                    cigarElems.add(cigarElement);
                                    elemLen = 0;
				    break;
                            case 'I':
                                cigarElemsLen.add(elemLen);
                                cigarElems.add(cigarElement);
                                accumCigarToTramscriptLength += elemLen;
        			 accumExonLength += elemLen; //although not a part of an exon insertion length has to be accounted for on the transcript
                                elemLen = 0;
                                break;
                        }
			   boolean moreInCigar = false; //not needed if elemLem > 0			
			   if (elemLen == 0) 
				if (cigarIndex < cigarLength) {
					moreInCigar = true;
					int i = cigarIndex;
		                     while ((i < cigarLength) && (Character.isDigit(transcriptomeCigar.charAt(i))))
                      	              	i++;
                                     if ((transcriptomeCigar.charAt(i)== 'H') || (transcriptomeCigar.charAt(i)== 'S' ))
						moreInCigar = false;
				}
                        if ((accumCigarToTramscriptLength == accumExonLength) && ((elemLen > 0) || moreInCigar)) {
                            if (currentExon == exons.size()-1)  //last exon
                                    throw new IllegalArgumentException("cigar string extends beyond transcript" + transcriptomeCigar);
                        cigarElemsLen.add((int) (exons.getStart(currentExon + 1) - exons.getEnd(currentExon) - 1));
                        cigarElems.add('N');
                        currentExon++;
                        currentExonLength = exons.length(currentExon);
                        accumExonLength += currentExonLength;

                        }
                    }
                }
                return buildCigar(cigarElemsLen, cigarElems,isoform.getStrand(),BasesInPolyA);
	}

	//private int getReadLength(String cigar, int start, int end) {
        private int getReadLength(String cigar) {
//indels fix
// method updated to account for different cigar elements; however, read length is not currently used in creating the
// cigar for genome coordinate alignment as before
// it is only used to claculate the starting alignment position when the transcript strans is -ve.
// It should actually return the number of bases the read maps to on the transcript
// i.e number of matches; mismatches and deletions from the transcript

		int i = 0;//start;
		int n = cigar.length();//end;
                int elemLen;
                int readLen=0;
                int start = 0;


		while (i < n) {
			// StringBuilder cigarElemLength = new StringBuilder();
                    while ((i < n) && (Character.isDigit(cigar.charAt(i)))){
			//cigarElemLength.append(cigar.charAt(i));
                     i++;
                    }
			String temp=cigar.substring(start, i);
	              //elemLen=Utils.parseInt(cigar,10,start,i-1);
			//elemLen=Integer.parseInt(cigar.substring(start, i),10);
			elemLen=Integer.parseInt(temp,10);
                    if (i == n)
                        throw new IllegalArgumentException("Incorrect cigar string format" + cigar); //cigar.substring(start, end));
                    switch(cigar.charAt(i)){
                        case 'M':
                        case 'D':
                        case '=':
                        case 'X':
                            readLen+=elemLen;
	                     //elemLen+= Utils.parseInt(cigarElemLength.toString(),10,0,cigarElemLength.length()-1);
                            break;
                    }
                    i++;
                    start = i;

		}
		return readLen;
	}

	static class VisitorForGenomeCigar implements IntervalVisitor {
		private int lastEnd;
		private StringBuilder cigar;
//indels
                private String transcriptomeCigar;

		@Override
// method currently not used. cigar is built in  getGenomeCigar
		public void visit(int isoformIntervalStart, int isoformIntervalEnd,
				int genomeIntervalStart, int genomeIntervalEnd) {
			if (!(cigar.length() == 0)) {
				if (genomeIntervalStart < lastEnd) {
					throw new IllegalStateException("Genome interval start " + genomeIntervalStart
							+ " received after genome interval end " + lastEnd);
				}
				cigar.append((int) (genomeIntervalStart - lastEnd - 1));
				cigar.append('N');
			}
			lastEnd = genomeIntervalEnd;
			cigar.append((int) (genomeIntervalEnd - genomeIntervalStart + 1));
			cigar.append('M');

		}
//method added for handling indels in cigar
                public void setTranscriptomeCigar(String transcriptomeCigar) {
                    this.transcriptomeCigar = transcriptomeCigar;
                }

//method added for handling indels in cigar
                public String getTranscriptomeCigar() {
                    return(transcriptomeCigar);
                }
		public void setCigar(StringBuilder cigar) {
			this.cigar = cigar;
			this.lastEnd = 0;
		}
	}
}
